@@ -222,9 +222,9 @@ make_codeset_canonical_for_locale (const char  *name,
 {
         char *old_locale;
 
-        old_locale = setlocale (LC_CTYPE, name);
+        old_locale = setlocale (LC_CTYPE, NULL);
 
-        if (old_locale == NULL) {
+        if (setlocale (LC_CTYPE, name) == NULL) {
                 return;
         }
 
@@ -460,7 +460,7 @@ collect_locales_from_archive (void)
         locales_collected = TRUE;
  out:
 
-        g_mapped_file_free (mapped);
+        g_mapped_file_unref (mapped);
         return locales_collected;
 }
 
